@@ -3,17 +3,18 @@ use v6;
 
 use NativeCall;
 
+my constant libsnappy = 'snappy';
 constant SNAPPY_OK = 0;
 constant SNAPPY_INVALID_INPUT = 1;
 constant SNAPPY_BUFFER_TOO_SMALL = 2;
 
-sub snappy_max_compressed_length(Int) returns Int is native('libsnappy') {...}
-sub snappy_compress(CArray[uint8], Int, CArray[uint8], CArray[int]) returns Int is native('libsnappy') {...}
+sub snappy_max_compressed_length(size_t) returns size_t is native(libsnappy) {...}
+sub snappy_compress(CArray[uint8], size_t, CArray[uint8], size_t is rw) returns int32 is native(libsnappy) {...}
 
-sub snappy_uncompressed_length(CArray[uint8], Int, CArray[int]) returns Int is native('libsnappy') {...}
-sub snappy_uncompress(CArray[uint8], Int, CArray[uint8], CArray[int]) returns Int is native('libsnappy') {...}
+sub snappy_uncompressed_length(CArray[uint8], size_t, size_t is rw) returns int32 is native(libsnappy) {...}
+sub snappy_uncompress(CArray[uint8], size_t, CArray[uint8], size_t is rw) returns int32 is native(libsnappy) {...}
 
-sub snappy_validate_compressed_buffer(CArray[uint8], Int) returns Int is native('libsnappy') {...}
+sub snappy_validate_compressed_buffer(CArray[uint8], size_t) returns int32 is native(libsnappy) {...}
 
 # helper functions to hide translations between Perl and C representations
 sub _zero_array(Int $count) {
@@ -44,20 +45,20 @@ our sub validate(Blob $blob) returns Bool {
 
 our proto compress($) {*};
 multi compress(Blob $blob) returns Buf {
-	my $max-size = snappy_max_compressed_length($blob.bytes);
+	my size_t $max-size = snappy_max_compressed_length($blob.bytes);
 
 	# Allocate an int pointer to store the length
-	my $output-real-size = _int_pointer($max-size);
 	my $output = _zero_array($max-size);
 	my $input = _copy_blob_to_array($blob);
+	my size_t $blob-size = $blob.bytes;
 
-	my $status = snappy_compress($input, $blob.bytes, $output, $output-real-size);
+	my $status = snappy_compress($input, $blob-size, $output, $max-size);
 	if $status {
 		die "snappy_compress internal error: $status";
 	}
 
 	# Copy everything into a Buf
-	return Buf.new: map {$output[$_]}, ^$output-real-size[0];
+	return Buf.new: map {$output[$_]}, ^$max-size;
 }
 
 multi compress(Str $str) returns Buf {
@@ -66,19 +67,20 @@ multi compress(Str $str) returns Buf {
 
 our sub decompress(Blob $blob) returns Buf {
 	# Allocate an int pointer to store the length
-	my $uncompressed-length = _int_pointer;
+	my size_t $uncompressed-length;
 	my $compressed = _copy_blob_to_array($blob);
+	my size_t $blob-size = $blob.bytes;
 
-	my $status1 = snappy_uncompressed_length($compressed, $blob.bytes, $uncompressed-length);
+	my $status1 = snappy_uncompressed_length($compressed, $blob-size, $uncompressed-length);
 	if $status1 {
 		die "snappy_uncompress internal error: $status1";
 	}
 
-	my $uncompressed = _zero_array($uncompressed-length[0]);
+	my $uncompressed = _zero_array($uncompressed-length);
 	my $status2 = snappy_uncompress($compressed, $blob.bytes, $uncompressed, $uncompressed-length);
 	if $status2 {
 		die "snappy_uncompress internal error: $status2";
 	}
 
-	return Buf.new: map {$uncompressed[$_]}, ^$uncompressed-length[0];
+	return Buf.new: map {$uncompressed[$_]}, ^$uncompressed-length;
 }
